#include "localview.h"
#include <sys/stat.h>
#include <util/message.h>

FTPLocalView::FTPLocalView(const os::Rect& cRect, const String& zName ) : os::IconDirectoryView(cRect,getenv("HOME"),os::CF_FOLLOW_ALL)
{
	/* zName is currently ignored because of a limitation with libsyllable (to be fixed in later version) */
	m_pcServer = NULL;
}

void FTPLocalView::SetServer( Server* pcServer )
{
	m_pcServer = pcServer;
}

Server* FTPLocalView::GetServer()
{
	return( m_pcServer );
}

void FTPLocalView::MouseMove( const Point& cPos, int nCode, uint32 nButtons, Message* pcData )
{
	/* HACK to make the IconDirView highlight a folder when something is dragged over it */
	int nCount, nType;
	if( pcData != NULL && pcData->GetNameInfo( "remote-file/path", &nType, &nCount ) == 0 && nType == T_STRING && nCount > 0 ) {
		/* We add a 'fake' "file/path" member to the message to trick the IconDirView into highlighting folders when something is dragged over them */
		pcData->AddString( "file/path", "" );
	}
	IconDirectoryView::MouseMove( cPos, nCode, nButtons, pcData );
}

void FTPLocalView::MouseUp( const Point& cPosition, uint32 nButtons, Message* pcData )
{
	if( pcData == NULL || m_pcServer == NULL ) {
		IconDirectoryView::MouseUp( cPosition, nButtons, pcData );
		return;
	}

	int nCount, nType;
	pcData->GetNameInfo( "remote-file/path", &nType, &nCount );
	if( nType != T_STRING || nCount == 0 ) {
		IconDirectoryView::MouseUp( cPosition, nButtons, pcData );
		return;
	}
	String zSourceServer;
	if( pcData->FindString( "server", &zSourceServer ) != 0 ) {
		IconDirectoryView::MouseUp( cPosition, nButtons, pcData );
		return;
	}
	StopScroll();
	
	if( zSourceServer != m_pcServer->GetServerAddress() ) {
		printf( "FTP client: dropped file is from another server!\n" );
		return;
	}
	
	/* Check if there is a directory icon under the mouse. If so, we transfer the file into that directory.
	   Otherwise, we transfer it into this directory. */
	Path cDestDir = Path( GetPath() );
	for( uint i = 0; i < GetIconCount(); i++ ) {
		if( Rect( GetIconPosition( i ), GetIconPosition( i ) + GetIconSize() ).DoIntersect( ConvertToView( cPosition ) ) ) {
			/* Found an icon under the mouse */
			DirectoryIconData* pcData = static_cast<DirectoryIconData*>( GetIconData( i ) );
			if( S_ISDIR( pcData->m_sStat.st_mode ) ) {
				/* Found a dir under the mouse. Add it to the destination path */
				cDestDir.Append( pcData->m_zPath.c_str() );
				break;
			}
		}
	}
	printf( "FTPLocalView: File dropped onto %s\n", cDestDir.GetPath().c_str() );
	
	/* Start transferring the files */
	String zSource;
	for( int i = 0; pcData->FindString( "remote-file/path", &zSource, i ) == 0; i++ ) {
		Path cDestPath = cDestDir;
		String zFileName;
		int n = zSource.str().find_last_of( '/' );
		if( n == String::npos ) { /* No '/' in the filename!? */
			zFileName = zSource;
		} else {
			zFileName = zSource.str().substr( n );
		}
		printf( "zFileName = '%s'\n", zFileName.c_str() );
		cDestPath.Append( zFileName );

		printf( " Drag & drop %s to %s\n", zSource.c_str(), cDestPath.GetPath().c_str() );
		
		/* Ask the Server object to start the transfer */
		m_pcServer->GetRemote( zSource, cDestPath.GetPath().c_str() );	 
	}	
}


