#ifndef __QUEUENODE_H__
#define __QUEUENODE_H__

#include <storage/file.h>
#include <util/string.h>
#include <util/handler.h>
#include <curl/curl.h>

using namespace os;

class Server;

/* Node status enum */
enum {
	NODE_QUEUED = 1,
	NODE_RUNNING = 2
};

enum {
	NODE_DOWNLOAD,
	NODE_UPLOAD,
	NODE_DIRLIST
};

class QueueNode
{
protected:
	QueueNode( Server* pcServer, const String& zLocalPath, const String& zRemotePath, int nType, int nID );

public:
	virtual ~QueueNode();	/* I want this to be private but can't be bothered debugging the compiler errors for now */
	
	/* Should these be here? (GUI access methods) */
	int GetStatus() { return( m_nStatus ); }
	int GetType() { return( m_nType ); }
	float GetProgress() { return( 0 ); } /* TODO */
	
private:
	virtual size_t Read( void* pBuf, size_t nSize ) { printf( "Warning: QueueNode::Read() called!\n" ); return( 0 ); }
	virtual size_t Write( void* pBuf, size_t nSize ) { printf( "Warning: QueueNode::Write() called!\n" ); return( 0 ); }
	virtual int Seek( curl_off_t nOffset, int nOrigin ) { printf( "Warning: QueueNode::Seek() called!\n" ); return( -1 ); }
	virtual int ProgressBar( double fDownTotal, double fDownNow, double fUpTotal, double fUpNow ) { printf( "Warning: QueneNode::ProgressBar() called!\n" ); return( 0 ); }

	static size_t ReadCallback( void* pBuf, size_t nSize, size_t nMult, void* pCookie );
	static size_t WriteCallback( void* pBuf, size_t nSize, size_t nMult, void* pCookie );
	static int SeekCallback( void* pCookie, curl_off_t nOffset, int nOrigin );
	static int ProgressBarCallback( void* pCookie, double fDownTotal, double fDownNow, double fUpTotal, double fUpNow );
	
protected:
	virtual CURLcode SetupCurlHandleForTransfer( CURL* pHandle );

	Server* m_pcServer;

	String m_zLocalPath;
	String m_zRemotePath;
	int m_nType;	/* Upload, download, dirlist */
	int m_nStatus;	/* active, queued */
	
	int m_nID;		/* Our id for the Server's std::map */
	
	friend class TransferThread;
};

class DownloadNode : public QueueNode
{
public:
	
private:
	DownloadNode( Server* pcServer, const String& zLocalPath, const String& zRemotePath, int nID );
	~DownloadNode();
	CURLcode SetupCurlHandleForTransfer( CURL* pHandle );

	size_t Write( void* pBuf, size_t nSize );
	int Seek( curl_off_t nOffset, int nOrigin );
	int ProgressBar( double fDownTotal, double fDownNow, double fUpTotal, double fUpNow );

	File* m_pcFile;

	friend class TransferThread;
};

class UploadNode : public QueueNode
{
private:
	UploadNode( Server* pcServer, const String& zLocalPath, const String& zRemotePath, int nID );
	~UploadNode();
	CURLcode SetupCurlHandleForTransfer( CURL* pHandle );
	
	size_t Read( void* pBuf, size_t nSize );
	int Seek( curl_off_t nOffset, int nOrigin );
	int ProgressBar( double fDownTotal, double fDownNow, double fUpTotal, double fUpNow );

	File* m_pcFile;

	friend class TransferThread;
};

class DirListNode : public QueueNode
{
private:
	DirListNode( Server* pcServer, const String& zRemotePath, Handler* pcTarget, int nID );
	~DirListNode();
	CURLcode SetupCurlHandleForTransfer( CURL* pHandle );
	
	size_t Write( void* pBuf, size_t nSize );
	int ProgressBar( double fDownTotal, double fDownNow, double fUpTotal, double fUpNow );
	
	Handler* m_pcTarget;
	
	bool m_bInitial;
	
	String m_zLastLineFragment;

	friend class TransferThread;
};

#endif	/* __QUEUENODE_H__ */

