#ifndef __REMOTEICONVIEW_H__
#define __REMOTEICONVIEW_H__

#include <gui/iconview.h>
#include <util/datetime.h>
#include <gui/image.h>
#include <gui/bitmap.h>
#include <gui/stringview.h>

#include <vector>

#include "application.h"
#include "messages.h"
#include "remotenode.h"
#include "server.h"



using namespace os;


class RemoteIconData : public IconData {
public:
	RemoteNode m_cNode;
};

class FTPRemoteView : public IconView
{
public:
	FTPRemoteView( const Rect& cFrame, const String& zName );
	~FTPRemoteView();
	
	void SetServer( Server* pcServer );
	Server* GetServer();
	void SetPath( const String& zPath );
	String GetPath();
	void ShowMessage( const String& zMsg );
	void HideMessage();
	
	void Update();	/* Asynchronously request a directory listing from the server via ServerConnection object */
	void SetContents( std::vector< RemoteNode >* pacContents, bool bInitial, bool bFinal );	/* Display the results of the directory listing; call Layout() if bFinal */
	

	void AttachedToWindow();
	void SetDirChangeMsg( Message* pcMsg );
	void Invoked( uint nIcon, IconData* pcData );	/* Browse to subdir, view file or start transfer */
	void MouseUp( const Point& cPoint, uint32 nButtons, Message* pcData );	/* Start transfer of dragged file */

	void DragSelection( Point cStartPoint );	/* Start drag & drop */
//	void OpenContextMenu( Point cPosition, bool bMouseOverIcon );	/* Context menu includes 'transfer', 'view', 'rename', 'delete' etc */
	
	void HandleMessage( Message* pcMessage );
	
	Image* GetNodeImage( RemoteNode* pcNode, bool bSmall );
	
private:
	Server* m_pcServer;	/* ServerConnection object provides GetChildren(), StartTransfer(), PauseTransfer(), CancelTransfer() etc */

	String m_zPath;		/* Path of current dir; absolute path from remote server root (eg /home/user ) */
	
	bool m_bUpdatePending;	/* If true, we are waiting for the server to give a directory listing, so show an hourglass instead */
	int m_nUpdateCookie;	/* Passed to ServerConnection::RequestDirListing(), so we don't get confused by late updates */ /* AWM: Is this necessary? */
	
	Message* m_pcDirChangedMsg;		/* Sent to the target when user changes dir */
	
//	StringView* m_pcStringView;		/* Used for displaying "Not connected" message, etc */	/* TODO: crashes when quitting !? */
};


#endif  /* __REMOTEICONVIEW_H__ */
