#ifndef __TRANSFERTHREAD_H__
#define __TRANSFERTHREAD_H__

#include <util/thread.h>
#include <util/message.h>
#include <util/locker.h>
#include <list>
#include <queue>

#include <curl/curl.h>

using namespace os;

class Server;

/* Status numbers for CurlHandle_s */
enum {
	HANDLE_IDLE = -1,
	HANDLE_INVALID = -2
};

typedef struct {
	CURL*	m_pHandle;
	int		m_nStatus;	/* If positive, it is the index of a QueueNode in the Server's list; if negative it is one of the enum above. */
} CurlHandle_s;

/* This is the thread in which all the transfers are done via calls to curl_multi_perform(). */
class TransferThread : public Thread
{
public:
	TransferThread( Server* pcServer );
	~TransferThread();
	
	/* Main thread code is in Run() */
	int32 Run();
	
	/* Lock/unlock the message list so no-one will change it while we are using it */
	int Lock();
	int Unlock();
	
	/* Send a message to the transfer thread asking it to add, start, etc a given transfer */
	int SendMessage( int nCode );
	int SendMessage( Message* pcMsg );
	
private:
	void _Notify();		/* Send a signal to the thread to notify it that it has new messages */
	
	void _ProcessMessages();	/* Process the messages in the message list */
	
	void _Schedule();	/* Assign queued transfers from the queue to idle curl handles */
	
	void _CleanupNode( int nNode );		/* Remove & delete a finished queue node */
	
	/* Our server object */
	Server* m_pcServer;
	
	/* Inter-thread communication */
	int m_nReadPipe;	/* Handles for the pipe for communication */
	int m_nWritePipe;
	std::queue< Message* > m_apcMessages;		/* Could use std::pair< Transfer*, int > instead? */
	Locker* m_pcLock;
	
	/* libcurl stuff */
	CURLM* m_pCurlMultiHandle;
	std::vector< CurlHandle_s* > m_asHandles;	/* List of all the curl handles we own, and the corresponding queue nodes */
	
	friend class Server;
};



#endif	/* __TRANSFERTHREAD_H__ */

