#include "view.h"
#include "address_field.h"
#include <gui/layoutview.h>


FTPView::FTPView(const os::Rect& cRect,const os::String& cName) : os::View(cRect,cName, os::CF_FOLLOW_ALL)
{
	remote = new FTPRemoteView(cRect, "remote_view");
	remote->SetView(os::IconView::VIEW_DETAILS);
	remote->SetDirChangeMsg( new Message( M_REMOTE_DIR_CHANGED ) );
	
	iconview = new FTPLocalView( cRect, "local_view" );		/* The name parameter currently isn't used because of libsyllable problems, but will be in future */
	iconview->SetView(os::IconDirectoryView::VIEW_DETAILS);
	iconview->SetDirChangeMsg(NULL);
	iconview->SetName("local_icon_dir");
	iconview->SetDirChangeMsg( new Message( M_LOCAL_DIR_CHANGED ) );
	
	iconview->SetTabOrder(100);	/* For keyboard navigation around the app */
	remote->SetTabOrder(101);
	
	m_pcLocalBar = new AddressField( iconview->GetPath(), new Message( M_LOCAL_PATH_BAR_INVOKED ) );
	m_pcRemoteBar = new AddressField( "", new Message( M_REMOTE_PATH_BAR_INVOKED ) );
	
	VLayoutNode* pcLeftRoot = new VLayoutNode( "left_pane_node" );
	HLayoutNode* pcTmp = new HLayoutNode( "left_top_node" );
	pcTmp->AddChild( m_pcLocalBar );		/* path bar */
	pcTmp->LimitMaxSize( Point( 1000000, 30 ) );
	pcLeftRoot->AddChild( pcTmp );
	pcLeftRoot->AddChild( iconview );
	LayoutView* pcLeft = new LayoutView( cRect, "left_pane", pcLeftRoot );
	
	VLayoutNode* pcRightRoot = new VLayoutNode( "right_pane_node" );
	HLayoutNode* pcTmp2 = new HLayoutNode( "right_top_node" );
	pcTmp2->AddChild( m_pcRemoteBar );		/* path bar */
	pcTmp2->LimitMaxSize( Point( 1000000, 30 ) );
	pcRightRoot->AddChild( pcTmp2 );
	pcRightRoot->AddChild( remote );
	LayoutView* pcRight = new LayoutView( cRect, "right_pane", pcRightRoot );
	
	m_pcSplitter = new os::Splitter(GetBounds(),"splitter",pcLeft,pcRight,os::HORIZONTAL,os::CF_FOLLOW_ALL);
	AddChild(m_pcSplitter);
}

void FTPView::HandleMessage( Message* pcMessage )
{
	switch( pcMessage->GetCode() )
	{
		case M_LOCAL_DIR_CHANGED:
		{
			printf( "LOCAL_DIR_CHANGED\n" );
			String zPath;
			if( pcMessage->FindString( "file/path", &zPath ) != 0 ) {
				printf( "FTPView: Got LOCAL_DIR_CHANGED without path parameter!\n" );
				break;
			}
			m_pcLocalBar->SetPath( zPath );
			break;
		}
		case M_REMOTE_DIR_CHANGED:
		{
			printf( "M_REMOTE_DIR_CHANGED\n" );
			String zPath;
			if( pcMessage->FindString( "file/path", &zPath ) != 0 ) {
				printf( "FTPView: Got REMOTE_DIR_CHANGED without path parameter!\n" );
				break;
			}
			m_pcRemoteBar->SetPath( zPath );
			break;
		}
		case M_LOCAL_PATH_BAR_INVOKED:
		{
			printf( "Local bar invoked\n" );
			String zPath;
			if( pcMessage->FindString( "file/path", &zPath ) != 0 ) {
				printf( "FTPView: Got LOCAL_PATH_BAR_INVOKED without file/path parameter!\n" );
				break;
			}
			iconview->SetPath( zPath );
			iconview->Clear();
			iconview->ReRead();
			m_pcLocalBar->SetPath( zPath );
			
			break;
		}
		case M_REMOTE_PATH_BAR_INVOKED:
		{
			printf( "Remote bar invoked\n" );
			String zPath;
			if( pcMessage->FindString( "file/path", &zPath ) != 0 ) {
				printf( "FTPView: Got REMOTE_PATH_BAR_INVOKED without file/path parameter!\n" );
				break;
			}
			remote->SetPath( zPath );
			m_pcRemoteBar->SetPath( zPath );
			break;
		}
		default:
		{
			View::HandleMessage( pcMessage );
		}
	}
}

void FTPView::SetServer( Server* pcServer )
{
	iconview->SetServer( pcServer );
	remote->SetServer( pcServer );
}

void FTPView::AllAttached()
{
	iconview->SetTarget(this);
	iconview->ReRead();
	
	remote->SetTarget(this);
	remote->Layout();

	m_pcLocalBar->SetTarget( this );
	m_pcRemoteBar->SetTarget( this );
}

os::Point FTPView::GetPreferredSize(bool) const
{
	return (os::Point(GetBounds().Width(),GetBounds().Height()));
}


void FTPView::ReRead()
{
}





